@@ -1,4 +1,3 @@
-// |jit-test| test-also-noasmjs
 load(libdir + "asm.js");
 load(libdir + "asserts.js");
 
@@ -70,8 +69,8 @@ assertEq(asmLink(asmCompile('glob', USE_ASM + 'var im=glob.Math.imul; function f
 
 var module = asmCompile('glob','i','b', USE_ASM + 'var i32=new glob.Int32Array(b); function f(){} return f');
 assertAsmLinkAlwaysFail(module, null, null);
-assertAsmLinkAlwaysFail(module, this, null, null);
-assertAsmLinkAlwaysFail(module, this, null, null);
+assertAsmLinkFail(module, this, null, null);
+assertAsmLinkFail(module, this, null, null);
 assertAsmLinkAlwaysFail(module, this, null, new ArrayBuffer(1));
 assertAsmLinkFail(module, this, null, new ArrayBuffer(4));
 assertAsmLinkFail(module, this, null, new ArrayBuffer(100));
@@ -139,6 +138,10 @@ assertTypeFailInEval('function *f() { "use asm"; function g() {} return g }');
 assertTypeFailInEval('f => { "use asm"; function g() {} return g }');
 assertTypeFailInEval('var f = { method() {"use asm"; return {}} }');
 assertAsmTypeFail(USE_ASM + 'return {m() {}};');
+assertTypeFailInEval('var f = { get p() {"use asm"; return {}} }');
+assertAsmTypeFail(USE_ASM + 'return {get p() {}};');
+assertTypeFailInEval('var f = { set p(x) {"use asm"; return {}} }');
+assertAsmTypeFail(USE_ASM + 'return {set p(x) {}};');
 assertTypeFailInEval('class f { constructor() {"use asm"; return {}} }');
 assertAsmTypeFail(USE_ASM + 'class c { constructor() {}}; return c;');
 
@@ -164,14 +167,14 @@ var g = asmLink(asmCompile('glob', 'ffis', USE_ASM + 'var f = ffis.f; function g
 assertEq(g(), 0);
 
 if (wasmIsSupported()) {
-    var h = Wasm.instantiateModule(wasmTextToBinary(`(module
+    var h = new WebAssembly.Instance(new WebAssembly.Module(wasmTextToBinary(`(module
         (import $f "imp" "f" (param i32) (result i32))
-        (func $h (result i32) (call_import $f (i32.const 1)))
+        (func $h (result i32) (call $f (i32.const 1)))
         (export "h" $h)
-    )`), {imp:{f}}).exports.h;
+    )`)), {imp:{f}}).exports.h;
     assertEq(h(), 0);
 
-    var i = Wasm.instantiateModule(wasmTextToBinary(`(module (func $i) (export "i" $i))`)).exports.i
+    var i = new WebAssembly.Instance(new WebAssembly.Module(wasmTextToBinary(`(module (func $i) (export "i" $i))`))).exports.i
     var j = asmLink(asmCompile('glob', 'ffis', USE_ASM + 'var i = ffis.i; function j() { return i(1)|0; } return j'), null, {i});
     assertEq(j(), 0);
 }
