@@ -17,7 +17,6 @@ test();
 
 function test()
 {
-  enterFunc ('test');
   printBugNumber(BUGNUMBER);
   printStatus (summary);
  
@@ -29,6 +28,4 @@ function test()
 
   actual = f + '';
   compareSource(expect, actual, summary);
-
-  exitFunc ('test');
 }
