@@ -19,10 +19,11 @@ info: >
 includes: [propertyHelper.js]
 ---*/
 
-assert.sameValue(Date.prototype.getYear.name, "getYear");
-
-verifyNotEnumerable(Date.prototype.getYear, "name");
-verifyNotWritable(Date.prototype.getYear, "name");
-verifyConfigurable(Date.prototype.getYear, "name");
+verifyProperty(Date.prototype.getYear, "name", {
+  enumerable: false,
+  writable: false,
+  configurable: true,
+  value: "getYear"
+});
 
 reportCompare(0, 0);
