@@ -22,10 +22,11 @@ info: >
 includes: [propertyHelper.js]
 ---*/
 
-assert.sameValue(String.prototype.strike.length, 0);
-
-verifyNotEnumerable(String.prototype.strike, "length");
-verifyNotWritable(String.prototype.strike, "length");
-verifyConfigurable(String.prototype.strike, "length");
+verifyProperty(String.prototype.strike, "length", {
+  enumerable: false,
+  writable: false,
+  configurable: true,
+  value: 0
+});
 
 reportCompare(0, 0);
