@@ -6,17 +6,17 @@ info: >
     The abtract operation ToPropertyDescriptor  is used to package the
     into a property desc. Step 10 of ToPropertyDescriptor throws a TypeError
     if the property desc ends up having a mix of accessor and data property elements.
-es5id: 15.2.3.6-3-8
+es5id: 15.2.3.6-3-11
 description: >
-    Object.defineProperty throws TypeError if getter is not callable
-    but not undefined (Null)(8.10.5 step 7.b)
+    Object.defineProperty throws TypeError if setter is not callable
+    but not undefined (Boolean)(8.10.5 step 8.b)
 ---*/
 
     var o = {};
     
-    // dummy getter
-    var getter = null;
-    var desc = { get: getter };
+    // dummy setter
+    var setter = true;
+    var desc = { set: setter };
 assert.throws(TypeError, function() {
       Object.defineProperty(o, "foo", desc);
 });
