@@ -7,7 +7,7 @@ description: Value shared by all realms
 info: >
   Unless otherwise specified, well-known symbols values are shared by all
   realms.
-features: [Symbol.replace]
+features: [cross-realm, Symbol.replace]
 ---*/
 
 var OSymbol = $262.createRealm().global.Symbol;
