@@ -3,12 +3,14 @@
 
 /*---
 es6id: 22.2.6
+esid: sec-properties-of-typedarray-prototype-objects
 description: >
   The prototype of Float32Array.prototype is %TypedArrayPrototype%.
 info: >
   The value of the [[Prototype]] internal slot of a TypedArray prototype
   object is the intrinsic object %TypedArrayPrototype% (22.2.3).
 includes: [testTypedArray.js]
+features: [TypedArray]
 ---*/
 
 assert.sameValue(Object.getPrototypeOf(Float32Array.prototype), TypedArray.prototype);
