@@ -25,8 +25,8 @@ info: >
   ...
   3. Let proto be ? Get(constructor, "prototype").
   ...
-features: [Reflect]
 includes: [testTypedArray.js]
+features: [Reflect, TypedArray]
 ---*/
 
 var newTarget = function() {}.bind(null);
