@@ -11,7 +11,7 @@ info: >
   3. Let numValue be ? ToNumber(value).
   ...
 includes: [testTypedArray.js]
-features: [Symbol]
+features: [Symbol, TypedArray]
 ---*/
 
 var s = Symbol("1");
