@@ -31,7 +31,7 @@ info: >
   7. If IsConstructor(S) is true, return S.
   8. Throw a TypeError exception.
 includes: [testTypedArray.js]
-features: [Symbol.species]
+features: [Symbol.species, TypedArray]
 ---*/
 
 testWithTypedArrayConstructors(function(TA) {
