@@ -6,6 +6,8 @@ info: >
     If B = 1110xxxx (n = 3) and C != 10xxxxxx (C - first of octets after B),
     throw URIError
 es5id: 15.1.3.2_A1.14_T4
+es6id: 18.2.6.3
+esid: sec-decodeuricomponent-encodeduricomponent
 description: Complex tests. B = [0xE0 - 0xEF], C = [0xC0, 0xFF]
 includes: [decimalToHexString.js]
 ---*/
@@ -16,37 +18,37 @@ var indexP;
 var indexO = 0;
 
 for (var indexB = 0xE0; indexB <= 0xEF; indexB++) {
-  count++; 
+  count++;
   var hexB = decimalToPercentHexString(indexB);
   var result = true;
   for (var indexC = 0xC0; indexC <= 0xFF; indexC++) {
     var hexC = decimalToPercentHexString(indexC);
     try {
       decodeURIComponent(hexB + "%A0" + hexC);
-    } catch (e) { 
-      if ((e instanceof URIError) === true) continue;                
+    } catch (e) {
+      if ((e instanceof URIError) === true) continue;
     }
     result = false;
   }
   if (result !== true) {
-    if (indexO === 0) { 
+    if (indexO === 0) {
       indexO = indexB;
     } else {
-      if ((indexB - indexP) !== 1) {             
+      if ((indexB - indexP) !== 1) {
         if ((indexP - indexO) !== 0) {
           var hexP = decimalToHexString(indexP);
           var hexO = decimalToHexString(indexO);
           $ERROR('#' + hexO + '-' + hexP + ' ');
-        } 
+        }
         else {
           var hexP = decimalToHexString(indexP);
           $ERROR('#' + hexP + ' ');
-        }  
+        }
         indexO = indexB;
-      }         
+      }
     }
     indexP = indexB;
-    errorCount++;       
+    errorCount++;
   }
 }
 
@@ -58,7 +60,7 @@ if (errorCount > 0) {
   } else {
     var hexP = decimalToHexString(indexP);
     $ERROR('#' + hexP + ' ');
-  }     
+  }
   $ERROR('Total error: ' + errorCount + ' bad Unicode character in ' + count + ' ');
 }
 
