@@ -3,43 +3,46 @@
 
 /*---
 info: >
-    If string.charAt(k) in [0x0080 - 0x07FF], return 2 octets (00000yyy
-    yyzzzzzz -> 110yyyyy 10zzzzzz)
-es5id: 15.1.3.4_A2.2_T1
-description: Complex tests, use RFC 3629
+    If string.charAt(k) in [0xD800 - 0xDBFF] and string.length = k + 1, throw
+    URIError
+es5id: 15.1.3.4_A1.2_T1
+es6id: 18.2.6.5
+esid: sec-encodeuricomponent-uricomponent
+description: Complex tests
 includes: [decimalToHexString.js]
 ---*/
 
 var errorCount = 0;
 var count = 0;
 var indexP;
-var indexO = 0; 
-l:
-for (var index = 0x0080; index <= 0x07FF; index++) {
-  count++;  
-  var hex1 = decimalToPercentHexString(0x0080 + (index & 0x003F));
-  var hex2 = decimalToPercentHexString(0x00C0 + (index & 0x07C0) / 0x0040);
-  var str = String.fromCharCode(index);
-  if (encodeURIComponent(str).toUpperCase() === hex2 + hex1) continue;
+var indexO = 0;
 
-  if (indexO === 0) { 
+for (var index = 0xD800; index <= 0xDBFF; index++) {
+  count++;
+  var hex = decimalToHexString(index);
+  try {
+    encodeURIComponent(String.fromCharCode(index));
+  } catch (e) {
+    if ((e instanceof URIError) === true) continue;
+  }
+  if (indexO === 0) {
     indexO = index;
   } else {
-    if ((index - indexP) !== 1) {             
+    if ((index - indexP) !== 1) {
       if ((indexP - indexO) !== 0) {
         var hexP = decimalToHexString(indexP);
         var hexO = decimalToHexString(indexO);
         $ERROR('#' + hexO + '-' + hexP + ' ');
-      } 
+      }
       else {
         var hexP = decimalToHexString(indexP);
         $ERROR('#' + hexP + ' ');
-      }  
+      }
       indexO = index;
-    }         
+    }
   }
   indexP = index;
-  errorCount++;    
+  errorCount++;
 }
 
 if (errorCount > 0) {
@@ -50,7 +53,7 @@ if (errorCount > 0) {
   } else {
     var hexP = decimalToHexString(indexP);
     $ERROR('#' + hexP + ' ');
-  }     
+  }
   $ERROR('Total error: ' + errorCount + ' bad Unicode character in ' + count + ' ');
 }
 
