@@ -1,193 +1,108 @@
-// file: nativeFunctionMatcher.js
-/**
- * This regex makes a best-effort determination that the tested string matches
- * the NativeFunction grammar production without requiring a correct tokeniser.
- */
-const NATIVE_FUNCTION_RE = /\bfunction\b[\s\S]*\([\s\S]*\)[\s\S]*\{[\s\S]*\[[\s\S]*\bnative\b[\s\S]+\bcode\b[\s\S]*\][\s\S]*\}/;
-
-// file: testBuiltInObject.js
-// Copyright 2012 Mozilla Corporation. All rights reserved.
+// file: fnGlobalObject.js
+// Copyright (C) 2017 Ecma International.  All rights reserved.
 // This code is governed by the BSD license found in the LICENSE file.
+/*---
+description: |
+    Produce a reliable global object
+---*/
 
-/**
- * @description Tests that obj meets the requirements for built-in objects
- *   defined by the introduction of chapter 15 of the ECMAScript Language Specification.
- * @param {Object} obj the object to be tested.
- * @param {boolean} isFunction whether the specification describes obj as a function.
- * @param {boolean} isConstructor whether the specification describes obj as a constructor.
- * @param {String[]} properties an array with the names of the built-in properties of obj,
- *   excluding length, prototype, or properties with non-default attributes.
- * @param {number} length for functions only: the length specified for the function
- *   or derived from the argument list.
- * @author Norbert Lindenberg
- */
-
-function testBuiltInObject(obj, isFunction, isConstructor, properties, length) {
-
-  if (obj === undefined) {
-    $ERROR("Object being tested is undefined.");
-  }
-
-  var objString = Object.prototype.toString.call(obj);
-  if (isFunction) {
-    if (objString !== "[object Function]") {
-      $ERROR("The [[Class]] internal property of a built-in function must be " +
-          "\"Function\", but toString() returns " + objString);
-    }
-  } else {
-    if (objString !== "[object Object]") {
-      $ERROR("The [[Class]] internal property of a built-in non-function object must be " +
-          "\"Object\", but toString() returns " + objString);
-    }
-  }
-
-  if (!Object.isExtensible(obj)) {
-    $ERROR("Built-in objects must be extensible.");
-  }
-
-  if (isFunction && Object.getPrototypeOf(obj) !== Function.prototype) {
-    $ERROR("Built-in functions must have Function.prototype as their prototype.");
-  }
-
-  if (isConstructor && Object.getPrototypeOf(obj.prototype) !== Object.prototype) {
-    $ERROR("Built-in prototype objects must have Object.prototype as their prototype.");
-  }
-
-  // verification of the absence of the [[Construct]] internal property has
-  // been moved to the end of the test
-
-  // verification of the absence of the prototype property has
-  // been moved to the end of the test
-
-  if (isFunction) {
-
-    if (typeof obj.length !== "number" || obj.length !== Math.floor(obj.length)) {
-      $ERROR("Built-in functions must have a length property with an integer value.");
-    }
-
-    if (obj.length !== length) {
-      $ERROR("Function's length property doesn't have specified value; expected " +
-        length + ", got " + obj.length + ".");
-    }
-
-    var desc = Object.getOwnPropertyDescriptor(obj, "length");
-    if (desc.writable) {
-      $ERROR("The length property of a built-in function must not be writable.");
-    }
-    if (desc.enumerable) {
-      $ERROR("The length property of a built-in function must not be enumerable.");
-    }
-    if (!desc.configurable) {
-      $ERROR("The length property of a built-in function must be configurable.");
-    }
-  }
-
-  properties.forEach(function(prop) {
-    var desc = Object.getOwnPropertyDescriptor(obj, prop);
-    if (desc === undefined) {
-      $ERROR("Missing property " + prop + ".");
-    }
-    // accessor properties don't have writable attribute
-    if (desc.hasOwnProperty("writable") && !desc.writable) {
-      $ERROR("The " + prop + " property of this built-in object must be writable.");
-    }
-    if (desc.enumerable) {
-      $ERROR("The " + prop + " property of this built-in object must not be enumerable.");
-    }
-    if (!desc.configurable) {
-      $ERROR("The " + prop + " property of this built-in object must be configurable.");
-    }
-  });
-
-  // The remaining sections have been moved to the end of the test because
-  // unbound non-constructor functions written in JavaScript cannot possibly
-  // pass them, and we still want to test JavaScript implementations as much
-  // as possible.
-
-  var exception;
-  if (isFunction && !isConstructor) {
-    // this is not a complete test for the presence of [[Construct]]:
-    // if it's absent, the exception must be thrown, but it may also
-    // be thrown if it's present and just has preconditions related to
-    // arguments or the this value that this statement doesn't meet.
-    try {
-      /*jshint newcap:false*/
-      var instance = new obj();
-    } catch (e) {
-      exception = e;
-    }
-    if (exception === undefined || exception.name !== "TypeError") {
-      $ERROR("Built-in functions that aren't constructors must throw TypeError when " +
-        "used in a \"new\" statement.");
-    }
-  }
-
-  if (isFunction && !isConstructor && obj.hasOwnProperty("prototype")) {
-    $ERROR("Built-in functions that aren't constructors must not have a prototype property.");
-  }
-
-  // passed the complete test!
-  return true;
+var __globalObject = Function("return this;")();
+function fnGlobalObject() {
+  return __globalObject;
 }
 
-// file: proxyTrapsHelper.js
-function allowProxyTraps(overrides) {
-  function throwTest262Error(msg) {
-    return function () { throw new Test262Error(msg); };
-  }
-  if (!overrides) { overrides = {}; }
-  return {
-    getPrototypeOf: overrides.getPrototypeOf || throwTest262Error('[[GetPrototypeOf]] trap called'),
-    setPrototypeOf: overrides.setPrototypeOf || throwTest262Error('[[SetPrototypeOf]] trap called'),
-    isExtensible: overrides.isExtensible || throwTest262Error('[[IsExtensible]] trap called'),
-    preventExtensions: overrides.preventExtensions || throwTest262Error('[[PreventExtensions]] trap called'),
-    getOwnPropertyDescriptor: overrides.getOwnPropertyDescriptor || throwTest262Error('[[GetOwnProperty]] trap called'),
-    has: overrides.has || throwTest262Error('[[HasProperty]] trap called'),
-    get: overrides.get || throwTest262Error('[[Get]] trap called'),
-    set: overrides.set || throwTest262Error('[[Set]] trap called'),
-    deleteProperty: overrides.deleteProperty || throwTest262Error('[[Delete]] trap called'),
-    defineProperty: overrides.defineProperty || throwTest262Error('[[DefineOwnProperty]] trap called'),
-    enumerate: throwTest262Error('[[Enumerate]] trap called: this trap has been removed'),
-    ownKeys: overrides.ownKeys || throwTest262Error('[[OwnPropertyKeys]] trap called'),
-    apply: overrides.apply || throwTest262Error('[[Call]] trap called'),
-    construct: overrides.construct || throwTest262Error('[[Construct]] trap called')
-  };
-}
-
-// file: assertRelativeDateMs.js
-// Copyright (C) 2015 the V8 project authors. All rights reserved.
+// file: testTypedArray.js
+// Copyright (C) 2015 André Bargull. All rights reserved.
 // This code is governed by the BSD license found in the LICENSE file.
+/*---
+description: |
+    Collection of functions used to assert the correctness of TypedArray objects.
+---*/
+
 /**
- * Verify that the given date object's Number representation describes the
- * correct number of milliseconds since the Unix epoch relative to the local
- * time zone (as interpreted at the specified date).
+ * Array containing every typed array constructor.
+ */
+var typedArrayConstructors = [
+  Float64Array,
+  Float32Array,
+  Int32Array,
+  Int16Array,
+  Int8Array,
+  Uint32Array,
+  Uint16Array,
+  Uint8Array,
+  Uint8ClampedArray
+];
+
+/**
+ * The %TypedArray% intrinsic constructor function.
+ */
+var TypedArray = Object.getPrototypeOf(Int8Array);
+
+/**
+ * Callback for testing a typed array constructor.
  *
- * @param {Date} date
- * @param {Number} expectedMs
+ * @callback typedArrayConstructorCallback
+ * @param {Function} Constructor the constructor object to test with.
  */
-function assertRelativeDateMs(date, expectedMs) {
-  var actualMs = date.valueOf();
-  var localOffset = date.getTimezoneOffset() * 60000;
 
-  if (actualMs - localOffset !== expectedMs) {
-    $ERROR(
-      'Expected ' + date + ' to be ' + expectedMs +
-      ' milliseconds from the Unix epoch'
-    );
+/**
+ * Calls the provided function for every typed array constructor.
+ *
+ * @param {typedArrayConstructorCallback} f - the function to call for each typed array constructor.
+ * @param {Array} selected - An optional Array with filtered typed arrays
+ */
+function testWithTypedArrayConstructors(f, selected) {
+  var constructors = selected || typedArrayConstructors;
+  for (var i = 0; i < constructors.length; ++i) {
+    var constructor = constructors[i];
+    try {
+      f(constructor);
+    } catch (e) {
+      e.message += " (Testing with " + constructor.name + ".)";
+      throw e;
+    }
   }
 }
 
+/**
+ * Helper for conversion operations on TypedArrays, the expected values
+ * properties are indexed in order to match the respective value for each
+ * TypedArray constructor
+ * @param  {Function} fn - the function to call for each constructor and value.
+ *                         will be called with the constructor, value, expected
+ *                         value, and a initial value that can be used to avoid
+ *                         a false positive with an equivalent expected value.
+ */
+function testTypedArrayConversions(byteConversionValues, fn) {
+  var values = byteConversionValues.values;
+  var expected = byteConversionValues.expected;
+
+  testWithTypedArrayConstructors(function(TA) {
+    var name = TA.name.slice(0, -5);
+
+    return values.forEach(function(value, index) {
+      var exp = expected[name][index];
+      var initial = 0;
+      if (exp === 0) {
+        initial = 1;
+      }
+      fn(TA, value, exp, initial);
+    });
+  });
+}
+
 // file: byteConversionValues.js
 // Copyright (C) 2016 the V8 project authors. All rights reserved.
 // This code is governed by the BSD license found in the LICENSE file.
-
-/**
- * Provide a list for original and expected values for different byte
- * conversions.
- * This helper is mostly used on tests for TypedArray and DataView, and each
- * array from the expected values must match the original values array on every
- * index containing its original value.
- */
+/*---
+description: |
+    Provide a list for original and expected values for different byte
+    conversions.
+    This helper is mostly used on tests for TypedArray and DataView, and each
+    array from the expected values must match the original values array on every
+    index containing its original value.
+---*/
 var byteConversionValues = {
   values: [
     127,         // 2 ** 7 - 1
@@ -623,3 +538,374 @@ var byteConversionValues = {
     ]
   }
 };
+
+// file: nans.js
+// Copyright (C) 2017 Ecma International.  All rights reserved.
+// This code is governed by the BSD license found in the LICENSE file.
+/*---
+description: |
+    A collection of NaN values produced from expressions that have been observed
+    to create distinct bit representations on various platforms. These provide a
+    weak basis for assertions regarding the consistent canonicalization of NaN
+    values in Array buffers.
+---*/
+var distinctNaNs = [
+  0/0, Infinity/Infinity, -(0/0), Math.pow(-1, 0.5), -Math.pow(-1, 0.5)
+];
+
+// file: testBuiltInObject.js
+// Copyright 2012 Mozilla Corporation. All rights reserved.
+// This code is governed by the BSD license found in the LICENSE file.
+/*---
+description: |
+    A function used to assert the correctness of built-in objects.
+---*/
+
+/**
+ * @description Tests that obj meets the requirements for built-in objects
+ *   defined by the introduction of chapter 15 of the ECMAScript Language Specification.
+ * @param {Object} obj the object to be tested.
+ * @param {boolean} isFunction whether the specification describes obj as a function.
+ * @param {boolean} isConstructor whether the specification describes obj as a constructor.
+ * @param {String[]} properties an array with the names of the built-in properties of obj,
+ *   excluding length, prototype, or properties with non-default attributes.
+ * @param {number} length for functions only: the length specified for the function
+ *   or derived from the argument list.
+ * @author Norbert Lindenberg
+ */
+
+function testBuiltInObject(obj, isFunction, isConstructor, properties, length) {
+
+  if (obj === undefined) {
+    $ERROR("Object being tested is undefined.");
+  }
+
+  var objString = Object.prototype.toString.call(obj);
+  if (isFunction) {
+    if (objString !== "[object Function]") {
+      $ERROR("The [[Class]] internal property of a built-in function must be " +
+          "\"Function\", but toString() returns " + objString);
+    }
+  } else {
+    if (objString !== "[object Object]") {
+      $ERROR("The [[Class]] internal property of a built-in non-function object must be " +
+          "\"Object\", but toString() returns " + objString);
+    }
+  }
+
+  if (!Object.isExtensible(obj)) {
+    $ERROR("Built-in objects must be extensible.");
+  }
+
+  if (isFunction && Object.getPrototypeOf(obj) !== Function.prototype) {
+    $ERROR("Built-in functions must have Function.prototype as their prototype.");
+  }
+
+  if (isConstructor && Object.getPrototypeOf(obj.prototype) !== Object.prototype) {
+    $ERROR("Built-in prototype objects must have Object.prototype as their prototype.");
+  }
+
+  // verification of the absence of the [[Construct]] internal property has
+  // been moved to the end of the test
+
+  // verification of the absence of the prototype property has
+  // been moved to the end of the test
+
+  if (isFunction) {
+
+    if (typeof obj.length !== "number" || obj.length !== Math.floor(obj.length)) {
+      $ERROR("Built-in functions must have a length property with an integer value.");
+    }
+
+    if (obj.length !== length) {
+      $ERROR("Function's length property doesn't have specified value; expected " +
+        length + ", got " + obj.length + ".");
+    }
+
+    var desc = Object.getOwnPropertyDescriptor(obj, "length");
+    if (desc.writable) {
+      $ERROR("The length property of a built-in function must not be writable.");
+    }
+    if (desc.enumerable) {
+      $ERROR("The length property of a built-in function must not be enumerable.");
+    }
+    if (!desc.configurable) {
+      $ERROR("The length property of a built-in function must be configurable.");
+    }
+  }
+
+  properties.forEach(function(prop) {
+    var desc = Object.getOwnPropertyDescriptor(obj, prop);
+    if (desc === undefined) {
+      $ERROR("Missing property " + prop + ".");
+    }
+    // accessor properties don't have writable attribute
+    if (desc.hasOwnProperty("writable") && !desc.writable) {
+      $ERROR("The " + prop + " property of this built-in object must be writable.");
+    }
+    if (desc.enumerable) {
+      $ERROR("The " + prop + " property of this built-in object must not be enumerable.");
+    }
+    if (!desc.configurable) {
+      $ERROR("The " + prop + " property of this built-in object must be configurable.");
+    }
+  });
+
+  // The remaining sections have been moved to the end of the test because
+  // unbound non-constructor functions written in JavaScript cannot possibly
+  // pass them, and we still want to test JavaScript implementations as much
+  // as possible.
+
+  var exception;
+  if (isFunction && !isConstructor) {
+    // this is not a complete test for the presence of [[Construct]]:
+    // if it's absent, the exception must be thrown, but it may also
+    // be thrown if it's present and just has preconditions related to
+    // arguments or the this value that this statement doesn't meet.
+    try {
+      /*jshint newcap:false*/
+      var instance = new obj();
+    } catch (e) {
+      exception = e;
+    }
+    if (exception === undefined || exception.name !== "TypeError") {
+      $ERROR("Built-in functions that aren't constructors must throw TypeError when " +
+        "used in a \"new\" statement.");
+    }
+  }
+
+  if (isFunction && !isConstructor && obj.hasOwnProperty("prototype")) {
+    $ERROR("Built-in functions that aren't constructors must not have a prototype property.");
+  }
+
+  // passed the complete test!
+  return true;
+}
+
+// file: timer.js
+// Copyright (C) 2017 Ecma International.  All rights reserved.
+// This code is governed by the BSD license found in the LICENSE file.
+/*---
+description: |
+    Used in website/scripts/sth.js
+---*/
+//setTimeout is not available, hence this script was loaded
+if (Promise === undefined && this.setTimeout === undefined) {
+  if(/\$DONE()/.test(code))
+    $ERROR("Async test capability is not supported in your test environment");
+}
+
+if (Promise !== undefined && this.setTimeout === undefined) {
+  (function(that) {
+     that.setTimeout = function(callback, delay) {
+      var p = Promise.resolve();
+      var start = Date.now();
+      var end = start + delay;
+      function check(){
+        var timeLeft = end - Date.now();
+        if(timeLeft > 0)
+          p.then(check);
+        else
+          callback();
+      }
+      p.then(check);
+    }
+  })(this);
+}
+
+// file: proxyTrapsHelper.js
+// Copyright (C) 2017 Ecma International.  All rights reserved.
+// This code is governed by the BSD license found in the LICENSE file.
+/*---
+description: |
+    Used to assert the correctness of object behavior in the presence
+    and context of Proxy objects.
+---*/
+
+function allowProxyTraps(overrides) {
+  function throwTest262Error(msg) {
+    return function () { throw new Test262Error(msg); };
+  }
+  if (!overrides) { overrides = {}; }
+  return {
+    getPrototypeOf: overrides.getPrototypeOf || throwTest262Error('[[GetPrototypeOf]] trap called'),
+    setPrototypeOf: overrides.setPrototypeOf || throwTest262Error('[[SetPrototypeOf]] trap called'),
+    isExtensible: overrides.isExtensible || throwTest262Error('[[IsExtensible]] trap called'),
+    preventExtensions: overrides.preventExtensions || throwTest262Error('[[PreventExtensions]] trap called'),
+    getOwnPropertyDescriptor: overrides.getOwnPropertyDescriptor || throwTest262Error('[[GetOwnProperty]] trap called'),
+    has: overrides.has || throwTest262Error('[[HasProperty]] trap called'),
+    get: overrides.get || throwTest262Error('[[Get]] trap called'),
+    set: overrides.set || throwTest262Error('[[Set]] trap called'),
+    deleteProperty: overrides.deleteProperty || throwTest262Error('[[Delete]] trap called'),
+    defineProperty: overrides.defineProperty || throwTest262Error('[[DefineOwnProperty]] trap called'),
+    enumerate: throwTest262Error('[[Enumerate]] trap called: this trap has been removed'),
+    ownKeys: overrides.ownKeys || throwTest262Error('[[OwnPropertyKeys]] trap called'),
+    apply: overrides.apply || throwTest262Error('[[Call]] trap called'),
+    construct: overrides.construct || throwTest262Error('[[Construct]] trap called')
+  };
+}
+
+// file: decimalToHexString.js
+// Copyright (C) 2017 André Bargull. All rights reserved.
+// This code is governed by the BSD license found in the LICENSE file.
+/*---
+description: |
+    Collection of functions used to assert the correctness of various encoding operations.
+---*/
+
+function decimalToHexString(n) {
+  var hex = "0123456789ABCDEF";
+  n >>>= 0;
+  var s = "";
+  while (n) {
+    s = hex[n & 0xf] + s;
+    n >>>= 4;
+  }
+  while (s.length < 4) {
+    s = "0" + s;
+  }
+  return s;
+}
+
+function decimalToPercentHexString(n) {
+  var hex = "0123456789ABCDEF";
+  return "%" + hex[(n >> 4) & 0xf] + hex[n & 0xf];
+}
+
+// file: assertRelativeDateMs.js
+// Copyright (C) 2015 the V8 project authors. All rights reserved.
+// This code is governed by the BSD license found in the LICENSE file.
+/*---
+description: |
+    Verify that the given date object's Number representation describes the
+    correct number of milliseconds since the Unix epoch relative to the local
+    time zone (as interpreted at the specified date).
+---*/
+
+/**
+ * @param {Date} date
+ * @param {Number} expectedMs
+ */
+function assertRelativeDateMs(date, expectedMs) {
+  var actualMs = date.valueOf();
+  var localOffset = date.getTimezoneOffset() * 60000;
+
+  if (actualMs - localOffset !== expectedMs) {
+    $ERROR(
+      'Expected ' + date + ' to be ' + expectedMs +
+      ' milliseconds from the Unix epoch'
+    );
+  }
+}
+
+// file: arrayContains.js
+// Copyright (C) 2017 Ecma International.  All rights reserved.
+// This code is governed by the BSD license found in the LICENSE file.
+/*---
+description: |
+    Verify that a subArray is contained within an array.
+---*/
+
+/**
+ * @param {Array} array
+ * @param {Array} subArray
+ */
+
+function arrayContains(array, subArray) {
+  var found;
+  for (var i = 0; i < subArray.length; i++) {
+    found = false;
+    for (var j = 0; j < array.length; j++) {
+      if (subArray[i] === array[j]) {
+        found = true;
+        break;
+      }
+    }
+    if (!found) {
+      return false;
+    }
+  }
+  return true;
+}
+
+// file: tcoHelper.js
+// Copyright (C) 2015 André Bargull. All rights reserved.
+// This code is governed by the BSD license found in the LICENSE file.
+/*---
+description: |
+    This defines the number of consecutive recursive function calls that must be
+    made in order to prove that stack frames are properly destroyed according to
+    ES2015 tail call optimization semantics.
+---*/
+
+
+
+
+var $MAX_ITERATIONS = 100000;
+
+// file: promiseHelper.js
+// Copyright (C) 2017 Ecma International.  All rights reserved.
+// This code is governed by the BSD license found in the LICENSE file.
+/*---
+description: |
+    Check that an array contains a numeric sequence starting at 1
+    and incrementing by 1 for each entry in the array. Used by
+    Promise tests to assert the order of execution in deep Promise
+    resolution pipelines.
+---*/
+
+function checkSequence(arr, message) {
+  arr.forEach(function(e, i) {
+    if (e !== (i+1)) {
+      $ERROR((message ? message : "Steps in unexpected sequence:") +
+             " '" + arr.join(',') + "'");
+    }
+  });
+
+  return true;
+}
+
+// file: detachArrayBuffer.js
+// Copyright (C) 2017 Ecma International.  All rights reserved.
+// This code is governed by the BSD license found in the LICENSE file.
+/*---
+description: |
+    A function used in the process of asserting correctness of TypedArray objects.
+
+    $262.detachArrayBuffer is defined by a host.
+
+---*/
+
+function $DETACHBUFFER(buffer) {
+  if (!$262 || typeof $262.detachArrayBuffer !== "function") {
+    throw new Test262Error("No method available to detach an ArrayBuffer");
+  }
+  $262.detachArrayBuffer(buffer);
+}
+
+// file: dateConstants.js
+// Copyright (C) 2009 the Sputnik authors.  All rights reserved.
+// This code is governed by the BSD license found in the LICENSE file.
+/*---
+description: |
+    Collection of date-centric values
+---*/
+
+var date_1899_end = -2208988800001;
+var date_1900_start = -2208988800000;
+var date_1969_end = -1;
+var date_1970_start = 0;
+var date_1999_end = 946684799999;
+var date_2000_start = 946684800000;
+var date_2099_end = 4102444799999;
+var date_2100_start = 4102444800000;
+
+// file: nativeFunctionMatcher.js
+// Copyright (C) 2017 Ecma International.  All rights reserved.
+// This code is governed by the BSD license found in the LICENSE file.
+/*---
+description: |
+    This regex makes a best-effort determination that the tested string matches
+    the NativeFunction grammar production without requiring a correct tokeniser.
+---*/
+const NATIVE_FUNCTION_RE = /\bfunction\b[\s\S]*\([\s\S]*\)[\s\S]*\{[\s\S]*\[[\s\S]*\bnative\b[\s\S]+\bcode\b[\s\S]*\][\s\S]*\}/;
