@@ -23,6 +23,7 @@
 #include "jsapi.h"
 #include "jscntxt.h"
 
+#include "ds/Fifo.h"
 #include "jit/Ion.h"
 #include "threading/ConditionVariable.h"
 #include "vm/MutexIDs.h"
@@ -56,8 +57,8 @@ enum class ParseTaskKind
 
 namespace wasm {
 
-class CompileTask;
-typedef Vector<CompileTask*, 0, SystemAllocPolicy> CompileTaskPtrVector;
+struct CompileTask;
+typedef Fifo<CompileTask*, 0, SystemAllocPolicy> CompileTaskPtrFifo;
 
 struct Tier2GeneratorTask
 {
@@ -107,8 +108,8 @@ class GlobalHelperThreadState
     IonBuilderVector ionWorklist_, ionFinishedList_, ionFreeList_;
 
     // wasm worklists.
-    wasm::CompileTaskPtrVector wasmWorklist_tier1_;
-    wasm::CompileTaskPtrVector wasmWorklist_tier2_;
+    wasm::CompileTaskPtrFifo wasmWorklist_tier1_;
+    wasm::CompileTaskPtrFifo wasmWorklist_tier2_;
     wasm::Tier2GeneratorTaskPtrVector wasmTier2GeneratorWorklist_;
 
     // Count of finished Tier2Generator tasks.
@@ -145,6 +146,7 @@ class GlobalHelperThreadState
     size_t maxIonCompilationThreads() const;
     size_t maxWasmCompilationThreads() const;
     size_t maxWasmTier2GeneratorThreads() const;
+    size_t maxPromiseHelperThreads() const;
     size_t maxParseThreads() const;
     size_t maxCompressionThreads() const;
     size_t maxGCHelperThreads() const;
@@ -200,7 +202,7 @@ class GlobalHelperThreadState
         return ionFreeList_;
     }
 
-    wasm::CompileTaskPtrVector& wasmWorklist(const AutoLockHelperThreadState&, wasm::CompileMode m) {
+    wasm::CompileTaskPtrFifo& wasmWorklist(const AutoLockHelperThreadState&, wasm::CompileMode m) {
         switch (m) {
           case wasm::CompileMode::Once:
           case wasm::CompileMode::Tier1:
@@ -477,12 +479,16 @@ CancelOffThreadWasmTier2Generator();
 bool
 StartOffThreadPromiseHelperTask(JSContext* cx, UniquePtr<PromiseHelperTask> task);
 
+bool
+StartOffThreadPromiseHelperTask(PromiseHelperTask* task);
+
 /*
  * Schedule an Ion compilation for a script, given a builder which has been
  * generated and read everything needed from the VM state.
  */
 bool
-StartOffThreadIonCompile(JSContext* cx, jit::IonBuilder* builder);
+StartOffThreadIonCompile(JSContext* cx, jit::IonBuilder* builder,
+                         const AutoLockHelperThreadState& lock);
 
 /*
  * Schedule deletion of Ion compilation data.
@@ -537,12 +543,6 @@ CancelOffThreadIonCompilesUsingNurseryPointers(JSRuntime* runtime)
     CancelOffThreadIonCompile(CompilationSelector(CompilationsUsingNursery{runtime}), true);
 }
 
-inline void
-CancelOffThreadIonCompile()
-{
-    CancelOffThreadIonCompile(CompilationSelector(AllCompilations()), false);
-}
-
 #ifdef DEBUG
 bool
 HasOffThreadIonCompile(JSCompartment* comp);
@@ -794,7 +794,10 @@ struct PromiseHelperTask : OffThreadPromiseTask
 
     // May be called in the absence of helper threads or off-thread promise
     // support to synchronously execute and resolve a PromiseTask.
-    void executeAndResolve(JSContext* cx);
+    //
+    // Warning: After this function returns, 'this' can be deleted at any time, so the
+    // caller must immediately return from the stream callback.
+    void executeAndResolveAndDestroy(JSContext* cx);
 };
 
 } /* namespace js */
