@@ -18,6 +18,10 @@
 
 #include "ds/LifoAlloc.h"
 
+#ifdef XP_WIN32
+#include "jswin.h"
+#endif
+
 using mozilla::PodCopy;
 
 namespace
@@ -450,6 +454,18 @@ Fprinter::put(const char* s, size_t len)
         reportOutOfMemory();
         return false;
     }
+#ifdef XP_WIN32
+    if ((file_ == stderr) && (IsDebuggerPresent())) {
+        UniqueChars buf(static_cast<char*>(js_malloc(len + 1)));
+        if (!buf) {
+            reportOutOfMemory();
+            return false;
+        }
+        PodCopy(buf.get(), s, len);
+        buf[len] = '\0';
+        OutputDebugStringA(buf.get());
+    }
+#endif
     return true;
 }
 
