@@ -28,6 +28,8 @@
 
 #include "vm/NativeObject.h"
 
+#include "gc/ObjectKind-inl.h"
+
 namespace js {
 
 template<typename To, typename From>
@@ -608,6 +610,18 @@ class ElementSpecific
     }
 };
 
+
+/* static */ gc::AllocKind
+js::TypedArrayObject::AllocKindForLazyBuffer(size_t nbytes)
+{
+    MOZ_ASSERT(nbytes <= INLINE_BUFFER_LIMIT);
+    if (nbytes == 0)
+        nbytes += sizeof(uint8_t);
+    size_t dataSlots = AlignBytes(nbytes, sizeof(Value)) / sizeof(Value);
+    MOZ_ASSERT(nbytes <= dataSlots * sizeof(Value));
+    return gc::GetGCObjectKind(FIXED_DATA_START + dataSlots);
+}
+
 } // namespace js
 
 #endif // vm_TypedArrayObject_inl_h
