@@ -145,13 +145,13 @@
 
     void EnableViewSource(nsHtml5Highlighter* aHighlighter);
 
-    void errStrayStartTag(nsIAtom* aName);
+    void errStrayStartTag(nsAtom* aName);
 
-    void errStrayEndTag(nsIAtom* aName);
+    void errStrayEndTag(nsAtom* aName);
 
-    void errUnclosedElements(int32_t aIndex, nsIAtom* aName);
+    void errUnclosedElements(int32_t aIndex, nsAtom* aName);
 
-    void errUnclosedElementsImplied(int32_t aIndex, nsIAtom* aName);
+    void errUnclosedElementsImplied(int32_t aIndex, nsAtom* aName);
 
     void errUnclosedElementsCell(int32_t aIndex);
 
@@ -173,7 +173,7 @@
 
     void errNonSpaceInNoscriptInHead();
 
-    void errFooBetweenHeadAndBody(nsIAtom* aName);
+    void errFooBetweenHeadAndBody(nsAtom* aName);
 
     void errStartTagWithoutDoctype();
 
@@ -181,15 +181,15 @@
 
     void errStartSelectWhereEndSelectExpected();
 
-    void errStartTagWithSelectOpen(nsIAtom* aName);
+    void errStartTagWithSelectOpen(nsAtom* aName);
 
-    void errBadStartTagInHead(nsIAtom* aName);
+    void errBadStartTagInHead(nsAtom* aName);
 
     void errImage();
 
     void errIsindex();
 
-    void errFooSeenWhenFooOpen(nsIAtom* aName);
+    void errFooSeenWhenFooOpen(nsAtom* aName);
 
     void errHeadingWhenHeadingOpen();
 
@@ -197,27 +197,27 @@
 
     void errNoCellToClose();
 
-    void errStartTagInTable(nsIAtom* aName);
+    void errStartTagInTable(nsAtom* aName);
 
     void errFormWhenFormOpen();
 
     void errTableSeenWhileTableOpen();
 
-    void errStartTagInTableBody(nsIAtom* aName);
+    void errStartTagInTableBody(nsAtom* aName);
 
     void errEndTagSeenWithoutDoctype();
 
     void errEndTagAfterBody();
 
-    void errEndTagSeenWithSelectOpen(nsIAtom* aName);
+    void errEndTagSeenWithSelectOpen(nsAtom* aName);
 
     void errGarbageInColgroup();
 
     void errEndTagBr();
 
-    void errNoElementToCloseButEndTagSeen(nsIAtom* aName);
+    void errNoElementToCloseButEndTagSeen(nsAtom* aName);
 
-    void errHtmlStartTagInForeignContext(nsIAtom* aName);
+    void errHtmlStartTagInForeignContext(nsAtom* aName);
 
     void errTableClosedWhileCaptionOpen();
 
@@ -227,17 +227,17 @@
 
     void errUnclosedChildrenInRuby();
 
-    void errStartTagSeenWithoutRuby(nsIAtom* aName);
+    void errStartTagSeenWithoutRuby(nsAtom* aName);
 
     void errSelfClosing();
 
     void errNoCheckUnclosedElementsOnStack();
 
-    void errEndTagDidNotMatchCurrentOpenElement(nsIAtom* aName, nsIAtom* aOther);
+    void errEndTagDidNotMatchCurrentOpenElement(nsAtom* aName, nsAtom* aOther);
 
-    void errEndTagViolatesNestingRules(nsIAtom* aName);
+    void errEndTagViolatesNestingRules(nsAtom* aName);
 
-    void errEndWithUnclosedElements(nsIAtom* aName);
+    void errEndWithUnclosedElements(nsAtom* aName);
 
     void MarkAsBroken(nsresult aRv);
 
