@@ -5,7 +5,7 @@
 #ifndef nsHtml5TreeOpExecutor_h
 #define nsHtml5TreeOpExecutor_h
 
-#include "nsIAtom.h"
+#include "nsAtom.h"
 #include "nsTraceRefcnt.h"
 #include "nsHtml5TreeOperation.h"
 #include "nsHtml5SpeculativeLoad.h"
@@ -196,11 +196,6 @@ class nsHtml5TreeOpExecutor final : public nsHtml5DocumentBuilder,
 
     void ComplainAboutBogusProtocolCharset(nsIDocument* aDoc);
 
-    bool IsComplete()
-    {
-      return !mParser;
-    }
-    
     bool HasStarted()
     {
       return mStarted;
@@ -225,7 +220,13 @@ class nsHtml5TreeOpExecutor final : public nsHtml5DocumentBuilder,
      * queue unconditionally. (This is for the main thread case.)
      */
     virtual void MoveOpsFrom(nsTArray<nsHtml5TreeOperation>& aOpQueue) override;
-    
+
+    void ClearOpQueue();
+
+    void RemoveFromStartOfOpQueue(size_t aNumberOfOpsToRemove);
+
+    inline size_t OpQueueLength() { return mOpQueue.Length(); }
+
     nsHtml5TreeOpStage* GetStage()
     {
       return &mStage;
