@@ -2,13 +2,15 @@
 # License, v. 2.0. If a copy of the MPL was not distributed with this file,
 # You can obtain one at http://mozilla.org/MPL/2.0/.
 
-import os
+from __future__ import absolute_import
+
 import platform
 
 from mozboot.base import BaseBootstrapper
+from mozboot.linux_common import StyloInstall
 
 
-class CentOSFedoraBootstrapper(BaseBootstrapper):
+class CentOSFedoraBootstrapper(StyloInstall, BaseBootstrapper):
     def __init__(self, distro, version, dist_id, **kwargs):
         BaseBootstrapper.__init__(self, **kwargs)
 
@@ -21,6 +23,7 @@ class CentOSFedoraBootstrapper(BaseBootstrapper):
         self.packages = [
             'autoconf213',
             'mercurial',
+            'which',
         ]
 
         self.browser_group_packages = [
@@ -42,7 +45,11 @@ class CentOSFedoraBootstrapper(BaseBootstrapper):
             'yasm',
         ]
 
-        self.mobile_android_packages = []
+        self.mobile_android_packages = [
+            'java-1.8.0-openjdk-devel',
+            # For downloading the Android SDK and NDK.
+            'wget',
+        ]
 
         if self.distro in ('CentOS', 'CentOS Linux'):
             self.group_packages += [
@@ -74,13 +81,6 @@ class CentOSFedoraBootstrapper(BaseBootstrapper):
                 'python-dbus',
             ]
 
-            self.mobile_android_packages += [
-                'java-1.8.0-openjdk-devel',
-                'ncurses-devel.i686',
-                'libstdc++.i686',
-                'zlib-devel.i686',
-            ]
-
     def install_system_packages(self):
         self.dnf_groupinstall(*self.group_packages)
         self.dnf_install(*self.packages)
@@ -92,16 +92,10 @@ class CentOSFedoraBootstrapper(BaseBootstrapper):
         self.ensure_browser_packages(artifact_mode=True)
 
     def install_mobile_android_packages(self):
-        if self.distro in ('CentOS', 'CentOS Linux'):
-            BaseBootstrapper.install_mobile_android_packages(self)
-        elif self.distro == 'Fedora':
-            self.install_fedora_mobile_android_packages()
+        self.ensure_mobile_android_packages(artifact_mode=False)
 
     def install_mobile_android_artifact_mode_packages(self):
-        if self.distro in ('CentOS', 'CentOS Linux'):
-            BaseBootstrapper.install_mobile_android_artifact_mode_packages(self)
-        elif self.distro == 'Fedora':
-            self.install_fedora_mobile_android_packages(artifact_mode=True)
+        self.ensure_mobile_android_packages(artifact_mode=True)
 
     def ensure_browser_packages(self, artifact_mode=False):
         # TODO: Figure out what not to install for artifact mode
@@ -115,38 +109,17 @@ class CentOSFedoraBootstrapper(BaseBootstrapper):
 
             self.run_as_root(['rpm', '-ivh', yasm])
 
-    def install_fedora_mobile_android_packages(self, artifact_mode=False):
-        import android
-
+    def ensure_mobile_android_packages(self, artifact_mode=False):
         # Install Android specific packages.
         self.dnf_install(*self.mobile_android_packages)
 
-        # Fetch Android SDK and NDK.
-        mozbuild_path = os.environ.get('MOZBUILD_STATE_PATH', os.path.expanduser(os.path.join('~', '.mozbuild')))
-        self.sdk_path = os.environ.get('ANDROID_SDK_HOME', os.path.join(mozbuild_path, 'android-sdk-linux'))
-        self.ndk_path = os.environ.get('ANDROID_NDK_HOME', os.path.join(mozbuild_path, 'android-ndk-r11b'))
-        self.sdk_url = 'https://dl.google.com/android/android-sdk_r24.0.1-linux.tgz'
-        self.ndk_url = android.android_ndk_url('linux')
-
-        android.ensure_android_sdk_and_ndk(path=mozbuild_path,
-                                           sdk_path=self.sdk_path, sdk_url=self.sdk_url,
-                                           ndk_path=self.ndk_path, ndk_url=self.ndk_url,
-                                           artifact_mode=artifact_mode)
-
-        # Most recent version of build-tools appears to be 23.0.1 on Fedora
-        packages = [p for p in android.ANDROID_PACKAGES if not p.startswith('build-tools')]
-        packages.append('build-tools-23.0.1')
-
-        # 3. We expect the |android| tool to be at
-        # ~/.mozbuild/android-sdk-linux/tools/android.
-        android_tool = os.path.join(self.sdk_path, 'tools', 'android')
-        android.ensure_android_packages(android_tool=android_tool, packages=packages)
+        from mozboot import android
+        android.ensure_android('linux', artifact_mode=artifact_mode,
+                               no_interactive=self.no_interactive)
 
     def suggest_mobile_android_mozconfig(self, artifact_mode=False):
-        import android
-        android.suggest_mozconfig(sdk_path=self.sdk_path,
-                                  ndk_path=self.ndk_path,
-                                  artifact_mode=artifact_mode)
+        from mozboot import android
+        android.suggest_mozconfig('linux', artifact_mode=artifact_mode)
 
     def suggest_mobile_android_artifact_mode_mozconfig(self):
         self.suggest_mobile_android_mozconfig(artifact_mode=True)
