@@ -2,6 +2,8 @@
 # License, v. 2.0. If a copy of the MPL was not distributed with this file,
 # You can obtain one at http://mozilla.org/MPL/2.0/.
 
+from __future__ import absolute_import
+
 from mozboot.base import BaseBootstrapper
 
 
@@ -13,7 +15,6 @@ class FreeBSDBootstrapper(BaseBootstrapper):
 
         self.packages = [
             'autoconf213',
-            'cargo',
             'gmake',
             'gtar',
             'mercurial',
@@ -28,7 +29,7 @@ class FreeBSDBootstrapper(BaseBootstrapper):
             'gconf2',
             'gtk2',
             'gtk3',
-            'libGL',
+            'llvm40',
             'pulseaudio',
             'v4l_compat',
             'yasm',
@@ -37,10 +38,6 @@ class FreeBSDBootstrapper(BaseBootstrapper):
         if not self.which('unzip'):
             self.packages.append('unzip')
 
-        # GCC 4.2 or Clang 3.4 in base are too old
-        if self.flavor == 'freebsd' and self.version < 11:
-            self.browser_packages.append('gcc')
-
     def pkg_install(self, *packages):
         command = ['pkg', 'install']
         if self.no_interactive:
@@ -62,5 +59,9 @@ class FreeBSDBootstrapper(BaseBootstrapper):
         # TODO: Figure out what not to install for artifact mode
         self.pkg_install(*self.browser_packages)
 
+    def ensure_stylo_packages(self, state_dir, checkout_root):
+        # Already installed as browser package
+        pass
+
     def upgrade_mercurial(self, current):
         self.pkg_install('mercurial')
