@@ -27,7 +27,7 @@ import mozpack.path as mozpath
 def package_fennec_apk(inputs=[], omni_ja=None, classes_dex=None,
                        lib_dirs=[],
                        assets_dirs=[],
-                       szip_assets_libs_with=None,
+                       features_dirs=[],
                        root_files=[],
                        verbose=False):
     jarrer = Jarrer(optimize=False)
@@ -53,27 +53,58 @@ def package_fennec_apk(inputs=[], omni_ja=None, classes_dex=None,
             jarrer.remove(path)
         jarrer.add(path, file, compress=compress)
 
+    for features_dir in features_dirs:
+        finder = FileFinder(features_dir)
+        for p, f in finder.find('**'):
+            add(mozpath.join('assets', 'features', p), f, False)
+
     for assets_dir in assets_dirs:
-        finder = FileFinder(assets_dir, find_executables=False)
+        finder = FileFinder(assets_dir)
         for p, f in finder.find('**'):
             compress = None  # Take default from Jarrer.
             if p.endswith('.so'):
                 # Asset libraries are special.
-                if szip_assets_libs_with:
-                    # We need to szip libraries before packing.  The file
-                    # returned by the finder is not yet opened.  When it is
-                    # opened, it will "see" the content updated by szip.
-                    subprocess.check_output([szip_assets_libs_with,
-                                             mozpath.join(finder.base, p)])
-
-                if f.open().read(4) == 'SeZz':
-                    # We need to store (rather than deflate) szipped libraries
-                    # (even if we don't szip them ourselves).
+                if f.open().read(5)[1:] == '7zXZ':
+                    print('%s is already compressed' % p)
+                    # We need to store (rather than deflate) compressed libraries
+                    # (even if we don't compress them ourselves).
                     compress = False
+                elif buildconfig.substs.get('XZ'):
+                    cmd = [buildconfig.substs.get('XZ'), '-zkf',
+                           mozpath.join(finder.base, p)]
+
+                    # For now, the mozglue XZStream ELF loader can only support xz files
+                    # with a single stream that contains a single block. In xz, there is no
+                    # explicit option to set the max block count. Instead, we force xz to use
+                    # single thread mode, which results in a single block.
+                    cmd.extend(['--threads=1'])
+
+                    bcj = None
+                    if buildconfig.substs.get('MOZ_THUMB2'):
+                        bcj = '--armthumb'
+                    elif buildconfig.substs.get('CPU_ARCH') == 'arm':
+                        bcj = '--arm'
+                    elif buildconfig.substs.get('CPU_ARCH') == 'x86':
+                        bcj = '--x86'
+
+                    if bcj:
+                        cmd.extend([bcj])
+                    # We need to explicitly specify the LZMA filter chain to ensure consistent builds
+                    # across platforms. Note that the dict size must be less then 16MiB per the hardcoded
+                    # value in mozglue/linker/XZStream.cpp. This is the default LZMA filter chain for for
+                    # xz-utils version 5.0. See:
+                    # https://github.com/xz-mirror/xz/blob/v5.0.0/src/liblzma/lzma/lzma_encoder_presets.c
+                    # https://github.com/xz-mirror/xz/blob/v5.0.0/src/liblzma/api/lzma/container.h#L31
+                    cmd.extend(['--lzma2=dict=8MiB,lc=3,lp=0,pb=2,mode=normal,nice=64,mf=bt4,depth=0'])
+                    print('xz-compressing %s with %s' % (p, ' '.join(cmd)))
+                    subprocess.check_output(cmd)
+                    os.rename(f.path + '.xz', f.path)
+                    compress = False
+
             add(mozpath.join('assets', p), f, compress=compress)
 
     for lib_dir in lib_dirs:
-        finder = FileFinder(lib_dir, find_executables=False)
+        finder = FileFinder(lib_dir)
         for p, f in finder.find('**'):
             add(mozpath.join('lib', p), f)
 
@@ -105,9 +136,8 @@ def main(args):
                         help='Optional lib/ dirs to pack into APK file.')
     parser.add_argument('--assets-dirs', nargs='*', default=[],
                         help='Optional assets/ dirs to pack into APK file.')
-    parser.add_argument('--szip-assets-libs-with', default=None,
-                        help='IN PLACE szip assets/**/*.so BEFORE packing '
-                        'into APK file using the given szip executable.')
+    parser.add_argument('--features-dirs', nargs='*', default=[],
+                        help='Optional features/ dirs to pack into APK file.')
     parser.add_argument('--root-files', nargs='*', default=[],
                         help='Optional files to pack into APK file root.')
     args = parser.parse_args(args)
@@ -121,7 +151,7 @@ def main(args):
                                 classes_dex=args.classes_dex,
                                 lib_dirs=args.lib_dirs,
                                 assets_dirs=args.assets_dirs,
-                                szip_assets_libs_with=args.szip_assets_libs_with,
+                                features_dirs=args.features_dirs,
                                 root_files=args.root_files,
                                 verbose=args.verbose)
     jarrer.copy(args.output)
