@@ -10,9 +10,12 @@ import unittest
 
 from mozunit import main
 
+from mozbuild import schedules
 from mozbuild.frontend.context import BugzillaComponent
-from mozbuild.frontend.reader import BuildReaderError
-from mozbuild.frontend.reader import BuildReader
+from mozbuild.frontend.reader import (
+    BuildReaderError,
+    BuildReader,
+)
 
 from mozbuild.test.common import MockConfig
 
@@ -42,11 +45,13 @@ class TestBuildReader(unittest.TestCase):
 
         return MockConfig(path, **kwargs)
 
-    def reader(self, name, enable_tests=False, **kwargs):
+    def reader(self, name, enable_tests=False, error_is_fatal=True, **kwargs):
         extra = {}
         if enable_tests:
             extra['ENABLE_TESTS'] = '1'
-        config = self.config(name, extra_substs=extra)
+        config = self.config(name,
+                             extra_substs=extra,
+                             error_is_fatal=error_is_fatal)
 
         return BuildReader(config, **kwargs)
 
@@ -239,6 +244,20 @@ class TestBuildReader(unittest.TestCase):
         self.assertIn('A moz.build file called the error() function.', str(e))
         self.assertIn('    Some error.', str(e))
 
+    def test_error_error_func_ok(self):
+        reader = self.reader('reader-error-error-func', error_is_fatal=False)
+
+        contexts = list(reader.read_topsrcdir())
+
+    def test_error_empty_list(self):
+        reader = self.reader('reader-error-empty-list')
+
+        with self.assertRaises(BuildReaderError) as bre:
+            list(reader.read_topsrcdir())
+
+        e = bre.exception
+        self.assertIn('Variable DIRS assigned an empty value.', str(e))
+
     def test_inheriting_variables(self):
         reader = self.reader('inheriting-variables')
 
@@ -416,7 +435,8 @@ class TestBuildReader(unittest.TestCase):
         ])
 
         expected = {
-            'default/module.js': set(['default/tests/xpcshell/**']),
+            'default/module.js': set(['default/tests/xpcshell/**',
+                                      'default/tests/reftests/**']),
         }
 
         for path, pattern_set in expected.items():
@@ -461,6 +481,27 @@ class TestBuildReader(unittest.TestCase):
         with self.assertRaises(BuildReaderError):
             reader.files_info(['foo.js'])
 
+    def test_schedules(self):
+        reader = self.reader('schedules')
+        info = reader.files_info(['somefile', 'foo.win', 'foo.osx', 'subd/aa.py', 'subd/yaml.py'])
+        # default: all exclusive, no inclusive
+        self.assertEqual(info['somefile']['SCHEDULES'].inclusive, [])
+        self.assertEqual(info['somefile']['SCHEDULES'].exclusive, schedules.EXCLUSIVE_COMPONENTS)
+        # windows-only
+        self.assertEqual(info['foo.win']['SCHEDULES'].inclusive, [])
+        self.assertEqual(info['foo.win']['SCHEDULES'].exclusive, ['windows'])
+        # osx-only
+        self.assertEqual(info['foo.osx']['SCHEDULES'].inclusive, [])
+        self.assertEqual(info['foo.osx']['SCHEDULES'].exclusive, ['macosx'])
+        # top-level moz.build specifies subd/**.py with an inclusive option
+        self.assertEqual(info['subd/aa.py']['SCHEDULES'].inclusive, ['py-lint'])
+        self.assertEqual(info['subd/aa.py']['SCHEDULES'].exclusive, schedules.EXCLUSIVE_COMPONENTS)
+        # Files('yaml.py') in subd/moz.build *overrides* Files('subdir/**.py')
+        self.assertEqual(info['subd/yaml.py']['SCHEDULES'].inclusive, ['yaml-lint'])
+        self.assertEqual(info['subd/yaml.py']['SCHEDULES'].exclusive, schedules.EXCLUSIVE_COMPONENTS)
+
+        self.assertEqual(set(info['subd/yaml.py']['SCHEDULES'].components),
+                         set(schedules.EXCLUSIVE_COMPONENTS + ['yaml-lint']))
 
 if __name__ == '__main__':
     main()
