@@ -72,17 +72,10 @@ private:
   UniqueProfilerBacktrace mStack;
 };
 
-#define DECL_STREAM_PAYLOAD_BASE  \
+#define DECL_STREAM_PAYLOAD \
   virtual void StreamPayload(SpliceableJSONWriter& aWriter, \
                              const mozilla::TimeStamp& aProcessStartTime, \
-                             UniqueStacks& aUniqueStacks) override
-
-// If the profiler is disabled then StreamPayload() will never be called.
-#ifdef MOZ_GECKO_PROFILER
-# define DECL_STREAM_PAYLOAD DECL_STREAM_PAYLOAD_BASE ;
-#else
-# define DECL_STREAM_PAYLOAD DECL_STREAM_PAYLOAD_BASE { MOZ_CRASH(); }
-#endif
+                             UniqueStacks& aUniqueStacks) override;
 
 class TracingMarkerPayload : public ProfilerMarkerPayload
 {
@@ -209,29 +202,6 @@ private:
   mozilla::TimeStamp mVsyncTimestamp;
 };
 
-class GPUMarkerPayload : public ProfilerMarkerPayload
-{
-public:
-  GPUMarkerPayload(const mozilla::TimeStamp& aCpuTimeStart,
-                   const mozilla::TimeStamp& aCpuTimeEnd,
-                   uint64_t aGpuTimeStart,
-                   uint64_t aGpuTimeEnd)
-    : ProfilerMarkerPayload(aCpuTimeStart, aCpuTimeEnd)
-    , mCpuTimeStart(aCpuTimeStart)
-    , mCpuTimeEnd(aCpuTimeEnd)
-    , mGpuTimeStart(aGpuTimeStart)
-    , mGpuTimeEnd(aGpuTimeEnd)
-  {}
-
-  DECL_STREAM_PAYLOAD
-
-private:
-  mozilla::TimeStamp mCpuTimeStart;
-  mozilla::TimeStamp mCpuTimeEnd;
-  uint64_t mGpuTimeStart;
-  uint64_t mGpuTimeEnd;
-};
-
 class GCSliceMarkerPayload : public ProfilerMarkerPayload
 {
 public:
