@@ -27,6 +27,7 @@ class nsThread;
 class nsWrapperCache;
 
 namespace mozilla {
+class AutoSlowOperation;
 
 class CycleCollectedJSRuntime;
 
@@ -66,6 +67,17 @@ struct CycleCollectorResults
   uint32_t mNumSlices;
 };
 
+class MicroTaskRunnable
+{
+public:
+  MicroTaskRunnable() {}
+  NS_INLINE_DECL_REFCOUNTING(MicroTaskRunnable)
+  virtual void Run(AutoSlowOperation& aAso) = 0;
+  virtual bool Suppressed() { return false; }
+protected:
+  virtual ~MicroTaskRunnable() {}
+};
+
 class CycleCollectedJSContext
   : public LinkedListElement<CycleCollectedJSContext>
 {
@@ -182,6 +194,9 @@ public:
   // microtask processor entry point
   void AfterProcessMicrotask();
 
+  // Check whether we need an idle GC task.
+  void IsIdleGCTaskNeeded();
+
   uint32_t RecursionDepth();
 
   // Run in stable state (call through nsContentUtils)
@@ -197,6 +212,44 @@ public:
   // Queue an async microtask to the current main or worker thread.
   virtual void DispatchToMicroTask(already_AddRefed<nsIRunnable> aRunnable);
 
+  // Call EnterMicroTask when you're entering JS execution.
+  // Usually the best way to do this is to use nsAutoMicroTask.
+  void EnterMicroTask()
+  {
+    ++mMicroTaskLevel;
+  }
+
+  void LeaveMicroTask()
+  {
+    if (--mMicroTaskLevel == 0) {
+      PerformMicroTaskCheckPoint();
+    }
+  }
+
+  bool IsInMicroTask()
+  {
+    return mMicroTaskLevel != 0;
+  }
+
+  uint32_t MicroTaskLevel()
+  {
+    return mMicroTaskLevel;
+  }
+
+  void SetMicroTaskLevel(uint32_t aLevel)
+  {
+    mMicroTaskLevel = aLevel;
+  }
+
+  void PerformMicroTaskCheckPoint();
+
+  void DispatchMicroTaskRunnable(already_AddRefed<MicroTaskRunnable> aRunnable);
+
+  bool IsInStableOrMetaStableState()
+  {
+    return mDoingStableStates;
+  }
+
   // Storage for watching rejected promises waiting for some client to
   // consume their rejection.
   // Promises in this list have been rejected in the last turn of the
@@ -237,6 +290,30 @@ private:
   bool mDoingStableStates;
 
   bool mDisableMicroTaskCheckpoint;
+
+  uint32_t mMicroTaskLevel;
+  std::queue<RefPtr<MicroTaskRunnable>> mPendingMicroTaskRunnables;
+
+  uint32_t mMicroTaskRecursionDepth;
+};
+
+class MOZ_STACK_CLASS nsAutoMicroTask
+{
+public:
+  nsAutoMicroTask()
+  {
+    CycleCollectedJSContext* ccjs = CycleCollectedJSContext::Get();
+    if (ccjs) {
+      ccjs->EnterMicroTask();
+    }
+  }
+  ~nsAutoMicroTask()
+  {
+    CycleCollectedJSContext* ccjs = CycleCollectedJSContext::Get();
+    if (ccjs) {
+      ccjs->LeaveMicroTask();
+    }
+  }
 };
 
 } // namespace mozilla
