@@ -85,7 +85,10 @@
 #include "nsWrapperCache.h"
 #include "nsStringBuffer.h"
 #include "GeckoProfiler.h"
+
+#ifdef MOZ_GECKO_PROFILER
 #include "ProfilerMarkerPayload.h"
+#endif
 
 #ifdef MOZ_CRASHREPORTER
 #include "nsExceptionHandler.h"
@@ -507,6 +510,7 @@ CycleCollectedJSRuntime::CycleCollectedJSRuntime(JSContext* aCx)
   : mGCThingCycleCollectorGlobal(sGCThingCycleCollectorGlobal)
   , mJSZoneCycleCollectorGlobal(sJSZoneCycleCollectorGlobal)
   , mJSRuntime(JS_GetRuntime(aCx))
+  , mHasPendingIdleGCTask(false)
   , mPrevGCSliceCallback(nullptr)
   , mPrevGCNurseryCollectionCallback(nullptr)
   , mJSHolderMap(256)
@@ -833,6 +837,7 @@ CycleCollectedJSRuntime::GCSliceCallback(JSContext* aContext,
   CycleCollectedJSRuntime* self = CycleCollectedJSRuntime::Get();
   MOZ_ASSERT(CycleCollectedJSContext::Get()->Context() == aContext);
 
+#ifdef MOZ_GECKO_PROFILER
   if (profiler_is_active()) {
     if (aProgress == JS::GC_CYCLE_END) {
       profiler_add_marker(
@@ -848,6 +853,7 @@ CycleCollectedJSRuntime::GCSliceCallback(JSContext* aContext,
                                          aDesc.sliceToJSON(aContext)));
     }
   }
+#endif
 
   if (aProgress == JS::GC_CYCLE_END &&
       JS::dbg::FireOnGarbageCollectionHookRequired(aContext)) {
@@ -930,8 +936,10 @@ CycleCollectedJSRuntime::GCNurseryCollectionCallback(JSContext* aContext,
 
   if (aProgress == JS::GCNurseryProgress::GC_NURSERY_COLLECTION_START) {
     self->mLatestNurseryCollectionStart = TimeStamp::Now();
-  } else if ((aProgress == JS::GCNurseryProgress::GC_NURSERY_COLLECTION_END) &&
-             profiler_is_active())
+  }
+#ifdef MOZ_GECKO_PROFILER
+  else if (aProgress == JS::GCNurseryProgress::GC_NURSERY_COLLECTION_END &&
+           profiler_is_active())
   {
     profiler_add_marker(
       "GCMinor",
@@ -939,6 +947,7 @@ CycleCollectedJSRuntime::GCNurseryCollectionCallback(JSContext* aContext,
                                        TimeStamp::Now(),
                                        JS::MinorGcToJSON(aContext)));
   }
+#endif
 
   if (self->mPrevGCNurseryCollectionCallback) {
     self->mPrevGCNurseryCollectionCallback(aContext, aProgress, aReason);
