@@ -9,14 +9,16 @@
 
 #include "nsComponentManager.h"
 #include "nsChromeRegistry.h"
+#include "mozilla/Attributes.h"
 #include "mozilla/FileLocation.h"
 
 void ParseManifest(NSLocationType aType, mozilla::FileLocation& aFile,
                    char* aBuf, bool aChromeOnly, bool aXPTOnly = false);
 
-void LogMessage(const char* aMsg, ...);
+void LogMessage(const char* aMsg, ...) MOZ_FORMAT_PRINTF(1, 2);
 
 void LogMessageWithContext(mozilla::FileLocation& aFile,
-                           uint32_t aLineNumber, const char* aMsg, ...);
+                           uint32_t aLineNumber, const char* aMsg, ...)
+  MOZ_FORMAT_PRINTF(3, 4);
 
 #endif // ManifestParser_h
