@@ -380,6 +380,11 @@ void Gecko_AssignCString(nsACString* aThis, const nsACString* aOther)
   aThis->Assign(*aOther);
 }
 
+void Gecko_TakeFromCString(nsACString* aThis, nsACString* aOther)
+{
+  aThis->Assign(mozilla::Move(*aOther));
+}
+
 void Gecko_AppendCString(nsACString* aThis, const nsACString* aOther)
 {
   aThis->Append(*aOther);
@@ -395,6 +400,11 @@ bool Gecko_FallibleAssignCString(nsACString* aThis, const nsACString* aOther)
   return aThis->Assign(*aOther, mozilla::fallible);
 }
 
+bool Gecko_FallibleTakeFromCString(nsACString* aThis, nsACString* aOther)
+{
+  return aThis->Assign(mozilla::Move(*aOther), mozilla::fallible);
+}
+
 bool Gecko_FallibleAppendCString(nsACString* aThis, const nsACString* aOther)
 {
   return aThis->Append(*aOther, mozilla::fallible);
@@ -425,6 +435,11 @@ void Gecko_AssignString(nsAString* aThis, const nsAString* aOther)
   aThis->Assign(*aOther);
 }
 
+void Gecko_TakeFromString(nsAString* aThis, nsAString* aOther)
+{
+  aThis->Assign(mozilla::Move(*aOther));
+}
+
 void Gecko_AppendString(nsAString* aThis, const nsAString* aOther)
 {
   aThis->Append(*aOther);
@@ -440,6 +455,11 @@ bool Gecko_FallibleAssignString(nsAString* aThis, const nsAString* aOther)
   return aThis->Assign(*aOther, mozilla::fallible);
 }
 
+bool Gecko_FallibleTakeFromString(nsAString* aThis, nsAString* aOther)
+{
+  return aThis->Assign(mozilla::Move(*aOther), mozilla::fallible);
+}
+
 bool Gecko_FallibleAppendString(nsAString* aThis, const nsAString* aOther)
 {
   return aThis->Append(*aOther, mozilla::fallible);
