@@ -90,9 +90,18 @@ SharedThreadPool::Get(const nsCString& aName, uint32_t aThreadLimit)
   ReentrantMonitorAutoEnter mon(*sMonitor);
   SharedThreadPool* pool = nullptr;
   nsresult rv;
-  if (!sPools->Get(aName, &pool)) {
+
+  if (auto entry = sPools->LookupForAdd(aName)) {
+    pool = entry.Data();
+    if (NS_FAILED(pool->EnsureThreadLimitIsAtLeast(aThreadLimit))) {
+      NS_WARNING("Failed to set limits on thread pool");
+    }
+  } else {
     nsCOMPtr<nsIThreadPool> threadPool(CreateThreadPool(aName));
-    NS_ENSURE_TRUE(threadPool, nullptr);
+    if (NS_WARN_IF(!threadPool)) {
+      sPools->Remove(aName); // XXX entry.Remove()
+      return nullptr;
+    }
     pool = new SharedThreadPool(aName, threadPool);
 
     // Set the thread and idle limits. Note that we don't rely on the
@@ -101,14 +110,18 @@ SharedThreadPool::Get(const nsCString& aName, uint32_t aThreadLimit)
     // with 4 threads rather than what we expected; so we'll have unexpected
     // behaviour.
     rv = pool->SetThreadLimit(aThreadLimit);
-    NS_ENSURE_SUCCESS(rv, nullptr);
+    if (NS_WARN_IF(NS_FAILED(rv))) {
+      sPools->Remove(aName); // XXX entry.Remove()
+      return nullptr;
+    }
 
     rv = pool->SetIdleThreadLimit(aThreadLimit);
-    NS_ENSURE_SUCCESS(rv, nullptr);
+    if (NS_WARN_IF(NS_FAILED(rv))) {
+      sPools->Remove(aName); // XXX entry.Remove()
+      return nullptr;
+    }
 
-    sPools->Put(aName, pool);
-  } else if (NS_FAILED(pool->EnsureThreadLimitIsAtLeast(aThreadLimit))) {
-    NS_WARNING("Failed to set limits on thread pool");
+    entry.OrInsert([pool] () { return pool; });
   }
 
   MOZ_ASSERT(pool);
@@ -143,7 +156,8 @@ NS_IMETHODIMP_(MozExternalRefCountType) SharedThreadPool::Release(void)
   // Dispatch an event to the main thread to call Shutdown() on
   // the nsIThreadPool. The Runnable here will add a refcount to the pool,
   // and when the Runnable releases the nsIThreadPool it will be deleted.
-  NS_DispatchToMainThread(NewRunnableMethod(mPool, &nsIThreadPool::Shutdown));
+  NS_DispatchToMainThread(NewRunnableMethod(
+    "nsIThreadPool::Shutdown", mPool, &nsIThreadPool::Shutdown));
 
   // Stabilize refcount, so that if something in the dtor QIs, it won't explode.
   mRefCnt = 1;
