@@ -33,10 +33,16 @@
 #include "nsString.h"
 #include "nsLiteralString.h"
 #include "nsReadableUtils.h"
+#include "mozilla/UniquePtrExtensions.h"
 #else
 #ifdef XP_MACOSX
 #include <crt_externs.h>
 #include <spawn.h>
+#endif
+#ifdef XP_UNIX
+#ifndef XP_MACOSX
+#include "base/process_util.h"
+#endif
 #include <sys/wait.h>
 #include <sys/errno.h>
 #endif
@@ -72,11 +78,12 @@ nsProcess::nsProcess()
   , mShutdown(false)
   , mBlocking(false)
   , mStartHidden(false)
+  , mNoShell(false)
   , mPid(-1)
   , mObserver(nullptr)
   , mWeakObserver(nullptr)
   , mExitValue(-1)
-#if !defined(XP_MACOSX)
+#if !defined(XP_UNIX)
   , mProcess(nullptr)
 #endif
 {
@@ -122,7 +129,7 @@ nsProcess::Init(nsIFile* aExecutable)
 
 
 #if defined(XP_WIN)
-// Out param `aWideCmdLine` must be PR_Freed by the caller.
+// Out param `aWideCmdLine` must be free()d by the caller.
 static int
 assembleCmdLine(char* const* aArgv, wchar_t** aWideCmdLine, UINT aCodePage)
 {
@@ -236,13 +243,16 @@ assembleCmdLine(char* const* aArgv, wchar_t** aWideCmdLine, UINT aCodePage)
 void
 nsProcess::Monitor(void* aArg)
 {
-  char stackBaseGuess;
-
   RefPtr<nsProcess> process = dont_AddRef(static_cast<nsProcess*>(aArg));
 
+#ifdef MOZ_GECKO_PROFILER
+  Maybe<AutoProfilerRegisterThread> registerThread;
+  if (!process->mBlocking) {
+    registerThread.emplace("RunProcess");
+  }
+#endif
   if (!process->mBlocking) {
     NS_SetCurrentThreadName("RunProcess");
-    profiler_register_thread("RunProcess", &stackBaseGuess);
   }
 
 #if defined(PROCESSMODEL_WINAPI)
@@ -267,7 +277,7 @@ nsProcess::Monitor(void* aArg)
     }
   }
 #else
-#ifdef XP_MACOSX
+#ifdef XP_UNIX
   int exitCode = -1;
   int status = 0;
   pid_t result;
@@ -291,7 +301,7 @@ nsProcess::Monitor(void* aArg)
   // Lock in case Kill or GetExitCode are called during this
   {
     MutexAutoLock lock(process->mLock);
-#if !defined(XP_MACOSX)
+#if !defined(XP_UNIX)
     process->mProcess = nullptr;
 #endif
     process->mExitValue = exitCode;
@@ -309,10 +319,6 @@ nsProcess::Monitor(void* aArg)
     NS_DispatchToMainThread(NewRunnableMethod(
       "nsProcess::ProcessComplete", process, &nsProcess::ProcessComplete));
   }
-
-  if (!process->mBlocking) {
-    profiler_unregister_thread();
-  }
 }
 
 void
@@ -466,46 +472,78 @@ nsProcess::RunProcess(bool aBlocking, char** aMyArgv, nsIObserver* aObserver,
 
 #if defined(PROCESSMODEL_WINAPI)
   BOOL retVal;
-  wchar_t* cmdLine = nullptr;
+  UniqueFreePtr<wchar_t> cmdLine;
 
   // |aMyArgv| is null-terminated and always starts with the program path. If
   // the second slot is non-null then arguments are being passed.
-  if (aMyArgv[1] && assembleCmdLine(aMyArgv + 1, &cmdLine,
-                                    aArgsUTF8 ? CP_UTF8 : CP_ACP) == -1) {
-    return NS_ERROR_FILE_EXECUTION_FAILED;
-  }
+  if (aMyArgv[1] || mNoShell) {
+    // Pass the executable path as argv[0] to the launched program when calling
+    // CreateProcess().
+    char** argv = mNoShell ? aMyArgv : aMyArgv + 1;
 
-  /* The SEE_MASK_NO_CONSOLE flag is important to prevent console windows
-   * from appearing. This makes behavior the same on all platforms. The flag
-   * will not have any effect on non-console applications.
-   */
+    wchar_t* assembledCmdLine = nullptr;
+    if (assembleCmdLine(argv, &assembledCmdLine,
+                        aArgsUTF8 ? CP_UTF8 : CP_ACP) == -1) {
+      return NS_ERROR_FILE_EXECUTION_FAILED;
+    }
+    cmdLine.reset(assembledCmdLine);
+  }
 
   // The program name in aMyArgv[0] is always UTF-8
   NS_ConvertUTF8toUTF16 wideFile(aMyArgv[0]);
 
-  SHELLEXECUTEINFOW sinfo;
-  memset(&sinfo, 0, sizeof(SHELLEXECUTEINFOW));
-  sinfo.cbSize = sizeof(SHELLEXECUTEINFOW);
-  sinfo.hwnd   = nullptr;
-  sinfo.lpFile = wideFile.get();
-  sinfo.nShow  = mStartHidden ? SW_HIDE : SW_SHOWNORMAL;
-  sinfo.fMask  = SEE_MASK_FLAG_DDEWAIT |
-                 SEE_MASK_NO_CONSOLE |
-                 SEE_MASK_NOCLOSEPROCESS;
+  if (mNoShell) {
+    STARTUPINFO startupInfo;
+    ZeroMemory(&startupInfo, sizeof(startupInfo));
+    startupInfo.cb = sizeof(startupInfo);
+    startupInfo.dwFlags = STARTF_USESHOWWINDOW;
+    startupInfo.wShowWindow = mStartHidden ? SW_HIDE : SW_SHOWNORMAL;
 
-  if (cmdLine) {
-    sinfo.lpParameters = cmdLine;
-  }
+    PROCESS_INFORMATION processInfo;
+    retVal = CreateProcess(/* lpApplicationName = */ wideFile.get(),
+                           /* lpCommandLine */ cmdLine.get(),
+                           /* lpProcessAttributes = */ NULL,
+                           /* lpThreadAttributes = */ NULL,
+                           /* bInheritHandles = */ FALSE,
+                           /* dwCreationFlags = */ 0,
+                           /* lpEnvironment = */ NULL,
+                           /* lpCurrentDirectory = */ NULL,
+                           /* lpStartupInfo = */ &startupInfo,
+                           /* lpProcessInformation */ &processInfo);
 
-  retVal = ShellExecuteExW(&sinfo);
-  if (!retVal) {
-    return NS_ERROR_FILE_EXECUTION_FAILED;
-  }
+    if (!retVal) {
+      return NS_ERROR_FILE_EXECUTION_FAILED;
+    }
 
-  mProcess = sinfo.hProcess;
+    CloseHandle(processInfo.hThread);
 
-  if (cmdLine) {
-    free(cmdLine);
+    mProcess = processInfo.hProcess;
+  } else {
+    SHELLEXECUTEINFOW sinfo;
+    memset(&sinfo, 0, sizeof(SHELLEXECUTEINFOW));
+    sinfo.cbSize = sizeof(SHELLEXECUTEINFOW);
+    sinfo.hwnd   = nullptr;
+    sinfo.lpFile = wideFile.get();
+    sinfo.nShow  = mStartHidden ? SW_HIDE : SW_SHOWNORMAL;
+
+    /* The SEE_MASK_NO_CONSOLE flag is important to prevent console windows
+     * from appearing. This makes behavior the same on all platforms. The flag
+     * will not have any effect on non-console applications.
+     */
+    sinfo.fMask  = SEE_MASK_FLAG_DDEWAIT |
+                   SEE_MASK_NO_CONSOLE |
+                   SEE_MASK_NOCLOSEPROCESS;
+
+    if (cmdLine) {
+      sinfo.lpParameters = cmdLine.get();
+    }
+
+    retVal = ShellExecuteExW(&sinfo);
+    if (!retVal) {
+      return NS_ERROR_FILE_EXECUTION_FAILED;
+    }
+
+    mProcess = sinfo.hProcess;
   }
 
   mPid = GetProcessId(mProcess);
@@ -537,6 +575,20 @@ nsProcess::RunProcess(bool aBlocking, char** aMyArgv, nsIObserver* aObserver,
   if (result != 0) {
     return NS_ERROR_FAILURE;
   }
+#elif defined(XP_UNIX)
+  base::file_handle_mapping_vector fdMap;
+  std::vector<std::string> argvVec;
+  for (char** arg = aMyArgv; *arg != nullptr; ++arg) {
+    argvVec.push_back(*arg);
+  }
+  pid_t newPid;
+  if (base::LaunchApp(argvVec, fdMap, false, &newPid)) {
+    static_assert(sizeof(pid_t) <= sizeof(int32_t),
+                  "mPid is large enough to hold a pid");
+    mPid = static_cast<int32_t>(newPid);
+  } else {
+    return NS_ERROR_FAILURE;
+  }
 #else
   mProcess = PR_CreateProcess(aMyArgv[0], aMyArgv, nullptr, nullptr);
   if (!mProcess) {
@@ -603,6 +655,20 @@ nsProcess::SetStartHidden(bool aStartHidden)
   return NS_OK;
 }
 
+NS_IMETHODIMP
+nsProcess::GetNoShell(bool* aNoShell)
+{
+  *aNoShell = mNoShell;
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsProcess::SetNoShell(bool aNoShell)
+{
+  mNoShell = aNoShell;
+  return NS_OK;
+}
+
 NS_IMETHODIMP
 nsProcess::GetPid(uint32_t* aPid)
 {
@@ -629,7 +695,7 @@ nsProcess::Kill()
     if (TerminateProcess(mProcess, 0) == 0) {
       return NS_ERROR_FAILURE;
     }
-#elif defined(XP_MACOSX)
+#elif defined(XP_UNIX)
     if (kill(mPid, SIGKILL) != 0) {
       return NS_ERROR_FAILURE;
     }
