@@ -27,11 +27,11 @@
 namespace detail {
 
 template<typename T>
-class ProxyReleaseEvent : public mozilla::Runnable
+class ProxyReleaseEvent : public mozilla::CancelableRunnable
 {
 public:
   ProxyReleaseEvent(const char* aName, already_AddRefed<T> aDoomed)
-  : Runnable(aName), mDoomed(aDoomed.take()) {}
+  : CancelableRunnable(aName), mDoomed(aDoomed.take()) {}
 
   NS_IMETHOD Run() override
   {
@@ -39,6 +39,11 @@ public:
     return NS_OK;
   }
 
+  nsresult Cancel() override
+  {
+    return Run();
+  }
+
   NS_IMETHOD GetName(nsACString& aName) override
   {
 #ifdef RELEASE_OR_BETA
