@@ -57,7 +57,7 @@ nsChromeTreeOwner::~nsChromeTreeOwner()
 
 //*****************************************************************************
 // nsChromeTreeOwner::nsISupports
-//*****************************************************************************   
+//*****************************************************************************
 
 NS_IMPL_ADDREF(nsChromeTreeOwner)
 NS_IMPL_RELEASE(nsChromeTreeOwner)
@@ -73,7 +73,7 @@ NS_INTERFACE_MAP_END
 
 //*****************************************************************************
 // nsChromeTreeOwner::nsIInterfaceRequestor
-//*****************************************************************************   
+//*****************************************************************************
 
 NS_IMETHODIMP nsChromeTreeOwner::GetInterface(const nsIID& aIID, void** aSink)
 {
@@ -105,7 +105,7 @@ NS_IMETHODIMP nsChromeTreeOwner::GetInterface(const nsIID& aIID, void** aSink)
 
 //*****************************************************************************
 // nsChromeTreeOwner::nsIDocShellTreeOwner
-//*****************************************************************************   
+//*****************************************************************************
 
 NS_IMETHODIMP
 nsChromeTreeOwner::ContentShellAdded(nsIDocShellTreeItem* aContentShell,
@@ -274,10 +274,10 @@ nsChromeTreeOwner::GetHasPrimaryContent(bool* aResult)
 
 //*****************************************************************************
 // nsChromeTreeOwner::nsIBaseWindow
-//*****************************************************************************   
+//*****************************************************************************
 
 NS_IMETHODIMP nsChromeTreeOwner::InitWindow(nativeWindow aParentNativeWindow,
-   nsIWidget* parentWidget, int32_t x, int32_t y, int32_t cx, int32_t cy)   
+   nsIWidget* parentWidget, int32_t x, int32_t y, int32_t cx, int32_t cy)
 {
    // Ignore widget parents for now.  Don't think those are a vaild thing to call.
    NS_ENSURE_SUCCESS(SetPositionAndSize(x, y, cx, cy, 0), NS_ERROR_FAILURE);
@@ -430,13 +430,13 @@ NS_IMETHODIMP nsChromeTreeOwner::SetFocus()
    return mXULWindow->SetFocus();
 }
 
-NS_IMETHODIMP nsChromeTreeOwner::GetTitle(char16_t** aTitle)
+NS_IMETHODIMP nsChromeTreeOwner::GetTitle(nsAString& aTitle)
 {
    NS_ENSURE_STATE(mXULWindow);
    return mXULWindow->GetTitle(aTitle);
 }
 
-NS_IMETHODIMP nsChromeTreeOwner::SetTitle(const char16_t* aTitle)
+NS_IMETHODIMP nsChromeTreeOwner::SetTitle(const nsAString& aTitle)
 {
    NS_ENSURE_STATE(mXULWindow);
    return mXULWindow->SetTitle(aTitle);
@@ -444,13 +444,13 @@ NS_IMETHODIMP nsChromeTreeOwner::SetTitle(const char16_t* aTitle)
 
 //*****************************************************************************
 // nsChromeTreeOwner::nsIWebProgressListener
-//*****************************************************************************   
+//*****************************************************************************
 
 NS_IMETHODIMP
 nsChromeTreeOwner::OnProgressChange(nsIWebProgress* aWebProgress,
                                     nsIRequest* aRequest,
                                     int32_t aCurSelfProgress,
-                                    int32_t aMaxSelfProgress, 
+                                    int32_t aMaxSelfProgress,
                                     int32_t aCurTotalProgress,
                                     int32_t aMaxTotalProgress)
 {
@@ -495,7 +495,7 @@ NS_IMETHODIMP nsChromeTreeOwner::OnLocationChange(nsIWebProgress* aWebProgress,
   return NS_OK;
 }
 
-NS_IMETHODIMP 
+NS_IMETHODIMP
 nsChromeTreeOwner::OnStatusChange(nsIWebProgress* aWebProgress,
                                   nsIRequest* aRequest,
                                   nsresult aStatus,
@@ -506,9 +506,9 @@ nsChromeTreeOwner::OnStatusChange(nsIWebProgress* aWebProgress,
 
 
 
-NS_IMETHODIMP 
-nsChromeTreeOwner::OnSecurityChange(nsIWebProgress *aWebProgress, 
-                                    nsIRequest *aRequest, 
+NS_IMETHODIMP
+nsChromeTreeOwner::OnSecurityChange(nsIWebProgress *aWebProgress,
+                                    nsIRequest *aRequest,
                                     uint32_t state)
 {
     return NS_OK;
@@ -516,11 +516,11 @@ nsChromeTreeOwner::OnSecurityChange(nsIWebProgress *aWebProgress,
 
 //*****************************************************************************
 // nsChromeTreeOwner: Helpers
-//*****************************************************************************   
+//*****************************************************************************
 
 //*****************************************************************************
 // nsChromeTreeOwner: Accessors
-//*****************************************************************************   
+//*****************************************************************************
 
 void nsChromeTreeOwner::XULWindow(nsXULWindow* aXULWindow)
 {
